@@ -25,6 +25,7 @@
 #include "vma.h"
 #include "bfd.h"
 #include "proc_parse.h"
+#include "fdinfo.h"
 #include "parasite.h"
 #include "cr_options.h"
 #include "sysfs_parse.h"
@@ -633,6 +634,12 @@ static int vma_list_add(struct vma_area *vma_area,
 		pages = vma_area_len(vma_area) / PAGE_SIZE;
 		vma_area_list->priv_size += pages;
 		vma_area_list->priv_longest = max(vma_area_list->priv_longest, pages);
+	} else if (vma_area_is(vma_area, VMA_ANON_SHARED)) {
+		unsigned long pages;
+
+		pages = vma_area_len(vma_area) / PAGE_SIZE;
+		vma_area_list->shared_longest =
+			max(vma_area_list->shared_longest, pages);
 	}
 
 	*prev_vfi = *vfi;
@@ -658,6 +665,7 @@ int parse_smaps(pid_t pid, struct vm_area_list *vma_area_list,
 	vma_area_list->nr_aios = 0;
 	vma_area_list->priv_longest = 0;
 	vma_area_list->priv_size = 0;
+	vma_area_list->shared_longest = 0;
 	INIT_LIST_HEAD(&vma_area_list->h);
 
 	f.fd = open_proc(pid, "smaps");
