@@ -832,7 +832,7 @@ int kerndat_tcp_repair_window()
 
 	sk = socket(AF_INET, SOCK_STREAM, 0);
 	if (sk < 0) {
-		pr_perror("Unable to create a netlink socket");
+		pr_perror("Unable to create inet socket");
 		return -1;
 	}
 
